open "item";;
open "fmap";;

(* Finite Maps functions implementations *)

(* fmsize : ('a -\ 'b)  -o Num *)
let fmsize = IFn (fun x -> match x with
    IFinMap xs -> INum (float_of_int (list_length xs))
  | _ -> raise (Failure "runtime type error: fmsizing a nonfm")
);;

(* fmmap : ('a -> 'b) ->  ('c -\ 'a) -o ('c -\ 'b)  *)
let fmmap = IFn (fun f -> IFn (fun default -> IFn (fun tim -> IFn (fun fin ->
  let rec fmap f1 b1 =  (match b1 with
      [] ->  []
    | (k,r)::xs ->  ((k,(iapp f1 r)) :: (fmap f1 xs))
    )
  in
  (match fin with
      IFinMap fs ->  IFinMap (fmap f fs)
    | _ -> raise (Failure "runtime error: fmmapping nonbag")
  )
))));;


(* groupby : !2 (kVal x rec) bag -o (kVal -\ (rec bag))  *)
let groupby = IFn (fun kdbag ->
  let rec insert ke re l = ( match l with
      [] -> [(ke, IBag [re])]
    | (k1,lr1)::tl -> (match lr1 with IBag llr ->
      (if ke == k1 then ((ke,(IBag (re::llr)))::tl)
	       else ((k1, (IBag llr)) :: (insert re ke tl)))
	|_ -> raise (Failure "runtime error: groupby analyzing  nonbag"))
  )
  in let rec group bl = (match bl with
      [] -> []
    | y::ys -> (match y with IPair (k,r) -> (insert k r (group ys))
	| _ -> raise (Failure "runtime error: groupby pairmapping nonpair"))
     )
     in
     (match kdbag with
	 IBag kb -> IFinMap (group kb)
       | _ -> raise (Failure "runtime error: groupby nonbag")
     )
);;


(* fmjoin : !2 ('a -\ 'b) x ('a -\ 'c) -o ('a -\ ('b x 'c))  *)
let fmjoin = IFn (fun pairfm ->
  let rec matching x g = match g with
      [] -> None
    | hd::tl -> (if fst(x) == fst(hd) then  Some((fst(x),IPair(snd(x),snd(hd))))
      else (matching x tl))
  in let rec joining f g = ( match f with
      [] -> []
    | hd::tl -> (match (matching hd g) with
	None -> (joining tl g)
	|Some(ret) -> (ret::(joining tl g))
    )
     )
     in
     (match pairfm with
	 IPair (pf1,pf2) -> (match pf1 with
	     IFinMap f -> (match pf2 with
		 IFinMap g -> IFinMap (joining f g)
	       | _ -> raise (Failure "runtime error: fmjoin analyzing nonfm")
	     )
	   | _ -> raise (Failure "runtime error: fmjoin fmmapping nonfm")
	 )
       | _ -> raise (Failure "runtime error: fmjoin pairmapping nonpair")
     )
);;
