#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#define panic(a...) do { fprintf(stderr, a); fprintf(stderr, "\n"); exit(1); } while (0)

#define TYPE_APACHE 1
#define TYPE_CENSUS 2
#define TYPE_POINTS 3

int main(int argc, char *argv[])
{
  if (argc != 4)
    panic("Syntax: dbconvert [--apache-log] <infile> <outfile>");

  int type = 0;
  if (!strcmp(argv[1], "--apache-log")) 
    type = TYPE_APACHE;
  else if (!strcmp(argv[1], "--points"))
    type = TYPE_POINTS;
  else if (!strcmp(argv[1], "--census"))
    type = TYPE_CENSUS;
  else
    panic("Unknown type: %s", argv[1]);

  FILE *infile = fopen(argv[2], "r");
  if (!infile)
    panic("Cannot read file %s", argv[2]);
    
  int h = open(argv[3], O_CREAT|O_TRUNC|O_RDWR, 0644);
  if (h<0)
    panic("Cannot write file %s", argv[3]);
  
  int numRows = 0;
  int numBytesPerRow = 0;
  
  switch (type) {
    case TYPE_APACHE: 
      numBytesPerRow = 800;
      break;
    case TYPE_POINTS:
      numBytesPerRow = 128;
      break;
    case TYPE_CENSUS:
      numBytesPerRow = 128;
      break;
    default:
      panic("Unknown type");
  }
  
  int wResult = write(h, &numRows, sizeof(numRows));
  wResult    =  write(h, &numBytesPerRow, sizeof(numBytesPerRow));
  
  char buffer[10000];
  while (fgets(buffer, sizeof(buffer), infile)) {
    numRows ++;
  
    const char *separators = " \t\n";
    if (type == TYPE_CENSUS)
      separators = " ,\n";
  
    char *arg[20];
    arg[0] = strtok(buffer, separators);
    for (int i=1; i<20; i++)
      arg[i] = strtok(NULL, separators);
    
    char outbuf[10000];
    memset(outbuf, 0, sizeof(outbuf));
    
    if (type == TYPE_APACHE) {
      sprintf(outbuf, "P%c", 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, strtok(arg[0], "."), 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, strtok(NULL, "."), 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, strtok(NULL, "."), 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, strtok(NULL, "."), 1);
      sprintf(&outbuf[strlen(outbuf)], "U%c", 1);
      for (int i=3; i<20; i++)
        if (arg[i])
          sprintf(&outbuf[strlen(outbuf)], "P%cS%c%s%c", 1, 1, arg[i], 1);
      strcat(outbuf, "U");
    } else if (type == TYPE_POINTS) {
      sprintf(outbuf, "P%cN%c%s%cN%c%s", 1, 1, arg[0], 1, 1, arg[1]);
    } else if (type == TYPE_CENSUS) {
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, arg[0], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cS%c%s%c", 1, 1, arg[1], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cS%c%s%c", 1, 1, arg[2], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cS%c%s%c", 1, 1, arg[3], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, arg[4], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, arg[5], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, arg[6], 1);
      sprintf(&outbuf[strlen(outbuf)], "P%cN%c%s.0%c", 1, 1, arg[7], 1);
      strcat(outbuf, "U");
    }
    
    if (strlen(outbuf) > (numBytesPerRow-1))
      panic("Output buffer too small");

    int w = write(h, &outbuf, numBytesPerRow);
    if (w<numBytesPerRow)
      panic("Cannot write to %s (w=%d/%d, %s)", argv[3], w, numBytesPerRow, strerror(errno));
  }
  
  lseek(h, 0, SEEK_SET);
  wResult = write(h, &numRows, sizeof(numRows));

  close(h);
  fclose(infile);
  
  return 0;
}
