from z3 import *

# We encode reals with infinity as a pair (Z3 Boolean expression, Z3
# Real expression)

# We use a class in order to overload +, *, <, ==
class MyRClass:
    def __init__(self, inf, val, name=""):
        self.inf = inf
        self.val = val
        self.name = name

    def __add__(self, other):
        other = _to_MyR(other)
        return MyRClass(Or(self.inf, other.inf), self.val + other.val)
    def __mul__(self, other):
        other = _to_MyR(other)
        return MyRClass(Or(self.inf, other.inf),
                        self.val * other.val)
        # return MyRClass(Or(
        #                    And(self.inf,Not(other.inf),other.val != RealVal(0.0)),
        #                    And(other.inf,Not(self.inf),self.val  != RealVal(0.0)),
        #                    And(self.inf, other.inf)
        #                 ),
        #                 self.val * other.val)
    def __eq__(self, other):
        other = _to_MyR(other)
        return Or(And(self.inf, other.inf),
                  And(Not(self.inf), Not(other.inf), self.val == other.val))
    def __ge__(self, other):
        other = _to_MyR(other)
        return Or(self.inf,
                  And(Not(self.inf), Not(other.inf), self.val >= other.val))
    def __ne__(self, other):
        return Not(Or(And(self.inf, other.inf),
                  And(Not(self.inf), Not(other.inf), self.val == other.val)))
    def __lt__(self, other):
        other = _to_MyR(other)
        return And(Not(self.inf),
                   Or(other.inf, self.val < other.val))

def MyR(name):
    # A MyR variable is encoded as a pair of variables name.inf and name.var
    return MyRClass(Bool('%s.inf' % name), Real('%s.val' % name), name)

def MyRVal(v):
    return MyRClass(BoolVal(False), RealVal(v))

def Inf():
    return MyRClass(BoolVal(True), RealVal(0))

def _to_MyR(v):
    if isinstance(v, MyRClass):
        return v
    elif isinstance(v, ArithRef):
        return MyRClass(BoolVal(False), v)
    else:
        return MyRVal(v)

