(*
  Fuzz compiler.

  Copyright: The University of Pennsylvania 2012
  License: ...

  Eclipse Output

*)

(* For now we only support pretty printing *)

open Syntax
open Constraint

open Format

(***********************************************************************)
(* Pretty printing for Eclipse *)

let rec pp_si_eclipse ppf s =
  match s with
  (* This will ocurr in dependent types *)
  | SiVar(_v)        -> fprintf ppf "Universal quantified variable not supported in Eclipse"
  | SiIntConst(i)    -> fprintf ppf "%d" i
  (* *)
  | SiEVar(v)        -> fprintf ppf "S%d" v.v_index
  | SiInfty          -> fprintf ppf "inf"
  | SiConst(flt)     -> fprintf ppf "%.1f" flt
  | SiAdd (si1, si2) -> fprintf ppf "%a + %a" pp_si_eclipse si1 pp_si_eclipse si2
  | SiMult(si1, si2) -> fprintf ppf "%a * %a" pp_si_eclipse si1 pp_si_eclipse si2

let rec pp_list_eclipse pp fmt l = match l with
    []         -> fprintf fmt ""
  | [cs]       -> fprintf fmt "%a" pp cs
  | csx :: csl -> fprintf fmt "%a,@;%a" pp csx (pp_list_eclipse pp) csl

let pp_si_cstr_elem_eclipse fmt si_cstr = match si_cstr with
    CSiEq(_, si1, si2) -> fprintf fmt "%a $= %a" pp_si_eclipse si1 pp_si_eclipse si2
  | CSiGe(_, si1, si2) -> fprintf fmt "%a $>= %a" pp_si_eclipse si1 pp_si_eclipse si2

let pp_si_cstr_eclipse = (pp_list_eclipse pp_si_cstr_elem_eclipse)

let cs_elem_fv cs = match cs with
  | CSiEq(_, si1, si2) -> VarSet.union (si_efv si1) (si_efv si2)
  | CSiGe(_, si1, si2) -> VarSet.union (si_efv si1) (si_efv si2)

let rec cs_free_vars cs = match cs with
  | []          -> VarSet.empty
  | (cs :: csl) -> VarSet.union (cs_elem_fv cs) (cs_free_vars csl)

(* *)
let pp_vinfo_e fmt v =
  fprintf fmt "(\"S%d\",S%d)" v.v_index v.v_index

let pp_var_list_eclipse fmt varset =
  fprintf fmt "@[<h>[%a]@]" (pp_list_eclipse pp_vinfo_e) (VarSet.elements varset)

let eclipse_prelude =
"
:- lib(ic).

split([],[]).
split([(_,V)|Vl], [V|R]) :- !,
   split(Vl, R).
"

let eclipse_after =
"
fuzz_yes :- !,
     print(\"Typechecking correct, a model exists!\"), nl.

fuzz_no :- !,
     print(\"Eclipse couldn't find a model! Sensitivities not correct\"), nl.

print_fuzz([]).
print_fuzz([V|Vl]) :-
     print(V),!,
     nl,
     print_fuzz(Vl).

fuzz :- fuzz_cs(L,V), locate(V, 0.1), V $:: [0..inf], !, fuzz_yes, print_fuzz(L), fuzz_yes.
fuzz :- fuzz_no.
"
let pp_si_store_eclipse fmt cs =
  let cs_fv = cs_free_vars cs  in
  fprintf fmt "%s" eclipse_prelude;
  fprintf fmt "fuzz_cs(L,V) :- L = %a, split(L,V), reals(V), V $:: [0..inf], @[<v>%a@]." pp_var_list_eclipse cs_fv pp_si_cstr_eclipse cs;
  fprintf fmt "%s" eclipse_after

