#!/usr/bin/perl

sub sprint { # a printing version of system()
    print @_;
    print "\n";
#    system @_;
    die if $? & 127 or ($? >> 8);
}

sub make_binary {
    my ($queryfz) = @_;
    sprint "make clean";
    sprint "../fuzz ../queries2/$queryfz.fz -o ./query.ml";
    sprint "make";
    sprint "mv runquery ./binaries/$queryfz";
}

sub make_dbs {
    my ($dbtype, $dbname) = @_;
    sprint "./dbconvert --$dbtype ../queries/$dbname ./dbs/$dbname";    
}    

sub run_bin {
    my ($binary, $database, $outputfile, @options) = @_;
    sprint "./binaries/$binary @options ./dbs/$database > /mnt/rd/$outputfile";
    sprint "killall $binary";
    sprint "killall camlrun";
    sprint "killall runquery";
    sprint "sleep 2";
}

make_binary("ipquery");
make_dbs("apache-log", "10000.weblog");
make_binary("kmeans");
make_dbs("points", "points.txt");
make_binary("income-diff");
make_dbs("census", "census.dat");

make_binary("evil-memory");
make_dbs("apache-log", "benign10000.weblog");
make_binary("evil-memory2");
make_binary("evil-range");
make_binary("evil-term");
make_binary("evil-census");
make_dbs("census", "census-miss.dat");
make_dbs("census", "census-hit.dat");

sprint "mount / -o remount,ro";

#figure 5
$its = 1;
for my $i (1..$its) {
  run_bin("ipquery", "10000.weblog", "fig5.weblog.green.$i");
  run_bin("ipquery", "10000.weblog", "fig5.weblog.yellow.$i", "--mode notimeout");
  run_bin("income-diff", "census-miss.dat", "fig5.census.green.$i");
  run_bin("income-diff", "census-miss.dat", "fig5.census.yellow.$i", "--mode notimeout");
  run_bin("kmeans", "points.txt", "fig5.kmeans.green.$i");
  run_bin("kmeans", "points.txt", "fig5.kmeans.yellow.$i", "--mode notimeout");
}

#figure 7
#$its = 102; #should be 102 for the paper; throw away the first 2
#for my $i (1..$its) {
#  run_bin("evil-range", "benign10000.weblog", "fig7.prot.$i");
#}

#table 4 
$its = 1; #should be 102 for the paper; throw away the first 2
for my $i (1..$its) {
  run_bin("evil-range", "10000.weblog", "table4.delay.hit.$i");
  run_bin("evil-range", "benign10000.weblog", "table4.delay.miss.$i");
  run_bin("evil-memory", "10000.weblog", "table4.gc.hit.$i");
  run_bin("evil-memory", "benign10000.weblog", "table4.gc.miss.$i");
  run_bin("evil-memory2", "10000.weblog", "table4.mem.hit.$i");
  run_bin("evil-memory2", "benign10000.weblog", "table4.mem.miss.$i");
  run_bin("evil-term", "10000.weblog", "table4.term.hit.$i");
  run_bin("evil-term", "benign10000.weblog", "table4.term.miss.$i");
  run_bin("evil-census", "census-hit.dat", "table4.census.hit.$i");
  run_bin("evil-census", "census-miss.dat", "table4.census.miss.$i");
}
sprint "mount / -o remount,rw";


sprint "cp /mnt/rd/* /home/narayana/usenixsec2011data/results/";
sprint "cp ../queries2/*.fz /home/narayana/usenixsec2011data/queries/";
sprint "cp ../queries/10000.weblog /home/narayana/usenixsec2011data/databases/";
sprint "cp ../queries/benign10000.weblog /home/narayana/usenixsec2011data/databases/";
sprint "cp ../queries/points.txt /home/narayana/usenixsec2011data/databases/";
sprint "cp ../queries/census-miss.dat /home/narayana/usenixsec2011data/databases/";
sprint "cp ../queries/census-hit.dat /home/narayana/usenixsec2011data/databases/";

__END__
